import mutagen.mp4

from mediafile.utils import Image

from .base import ListStorageStyle, SoundCheckStorageStyleMixin, StorageStyle


class MP4StorageStyle(StorageStyle):
    """A general storage style for MPEG-4 tags."""

    formats = ["MP4"]

    def serialize(self, value):
        value = super().serialize(value)
        if self.key.startswith("----:") and isinstance(value, str):
            value = value.encode("utf-8")
        return value


class MP4TupleStorageStyle(MP4StorageStyle):
    """A style for storing values as part of a pair of numbers in an
    MPEG-4 file.
    """

    def __init__(self, key, index=0, **kwargs):
        super().__init__(key, **kwargs)
        self.index = index

    def deserialize(self, mutagen_value):
        items = mutagen_value or []
        packing_length = 2
        return list(items) + [0] * (packing_length - len(items))

    def get(self, mutagen_file):
        value = super().get(mutagen_file)[self.index]
        if value == 0:
            # The values are always present and saved as integers. So we
            # assume that "0" indicates it is not set.
            return None
        else:
            return value

    def set(self, mutagen_file, value):
        if value is None:
            value = 0
        items = self.deserialize(self.fetch(mutagen_file))
        items[self.index] = int(value)
        self.store(mutagen_file, items)

    def delete(self, mutagen_file):
        if self.index == 0:
            super().delete(mutagen_file)
        else:
            self.set(mutagen_file, None)


class MP4ListStorageStyle(ListStorageStyle, MP4StorageStyle):
    pass


class MP4SoundCheckStorageStyle(SoundCheckStorageStyleMixin, MP4StorageStyle):
    def __init__(self, key, index=0, **kwargs):
        super().__init__(key, **kwargs)
        self.index = index


class MP4BoolStorageStyle(MP4StorageStyle):
    """A style for booleans in MPEG-4 files. (MPEG-4 has an atom type
    specifically for representing booleans.)
    """

    def get(self, mutagen_file):
        try:
            return mutagen_file[self.key]
        except KeyError:
            return None

    def get_list(self, mutagen_file):
        raise NotImplementedError("MP4 bool storage does not support lists")

    def set(self, mutagen_file, value):
        mutagen_file[self.key] = value

    def set_list(self, mutagen_file, values):
        raise NotImplementedError("MP4 bool storage does not support lists")


class MP4ImageStorageStyle(MP4ListStorageStyle):
    """Store images as MPEG-4 image atoms. Values are `Image` objects."""

    def __init__(self, **kwargs):
        super().__init__(key="covr", **kwargs)

    def deserialize(self, data):
        return Image(data)

    def serialize(self, image):
        if image.mime_type == "image/png":
            kind = mutagen.mp4.MP4Cover.FORMAT_PNG
        elif image.mime_type == "image/jpeg":
            kind = mutagen.mp4.MP4Cover.FORMAT_JPEG
        else:
            raise ValueError("MP4 files only supports PNG and JPEG images")
        return mutagen.mp4.MP4Cover(image.data, kind)



import os
import re
import sys
from pathlib import Path

MATCH_VERSION_LINE = re.compile(r"version = \W((\d+\.\d+)\.\d.*?)\W").fullmatch

pyproject = Path(__file__).parent.parent / "pyproject.toml"
version_line_match = next(
    filter(None, map(MATCH_VERSION_LINE, pyproject.read_text().splitlines()))
)
release, version = version_line_match.groups()

sys.path.insert(0, os.path.abspath(".."))

extensions = [
    "sphinx.ext.autodoc",
]

source_suffix = ".rst"
master_doc = "index"
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

project = "MediaFile"
copyright = "2016, the beets project"
author = "the beets project"

pygments_style = "sphinx"
htmlhelp_basename = "mediafiledoc"

# LaTeX output.
latex_documents = [
    (
        master_doc,
        "MediaFile.tex",
        "MediaFile Documentation",
        "the beets project",
        "manual",
    ),
]

# mapage output.
man_pages = [(master_doc, "mediafile", "MediaFile Documentation", [author], 1)]



import base64

import mutagen
import mutagen.flac

from mediafile.utils import Image

from .base import ListStorageStyle


class VorbisImageStorageStyle(ListStorageStyle):
    """Store images in Vorbis comments. Both legacy COVERART fields and
    modern METADATA_BLOCK_PICTURE tags are supported. Data is
    base64-encoded. Values are `Image` objects.
    """

    formats = ["OggOpus", "OggTheora", "OggSpeex", "OggVorbis", "OggFlac"]

    def __init__(self):
        super().__init__(key="metadata_block_picture")
        self.as_type = bytes

    def fetch(self, mutagen_file):
        images = []
        if "metadata_block_picture" not in mutagen_file:
            # Try legacy COVERART tags.
            if "coverart" in mutagen_file:
                for data in mutagen_file["coverart"]:
                    images.append(Image(base64.b64decode(data)))
            return images
        for data in mutagen_file["metadata_block_picture"]:
            try:
                pic = mutagen.flac.Picture(base64.b64decode(data))
            except (TypeError, AttributeError):
                continue
            images.append(Image(data=pic.data, desc=pic.desc, type=pic.type))
        return images

    def store(self, mutagen_file, image_data):
        # Strip all art, including legacy COVERART.
        if "coverart" in mutagen_file:
            del mutagen_file["coverart"]
        if "coverartmime" in mutagen_file:
            del mutagen_file["coverartmime"]
        super().store(mutagen_file, image_data)

    def serialize(self, image):
        """Turn a Image into a base64 encoded FLAC picture block."""
        pic = mutagen.flac.Picture()
        pic.data = image.data
        pic.type = image.type_index
        pic.mime = image.mime_type
        pic.desc = image.desc or ""

        # Encoding with base64 returns bytes on both Python 2 and 3.
        # Mutagen requires the data to be a Unicode string, so we decode
        # it before passing it along.
        return base64.b64encode(pic.write()).decode("ascii")



import enum

# Human-readable type names.
TYPES = {
    "mp3": "MP3",
    "aac": "AAC",
    "alac": "ALAC",
    "ogg": "OGG",
    "opus": "Opus",
    "flac": "FLAC",
    "ape": "APE",
    "wv": "WavPack",
    "mpc": "Musepack",
    "asf": "Windows Media",
    "aiff": "AIFF",
    "dsf": "DSD Stream File",
    "wav": "WAVE",
}


class ImageType(enum.Enum):
    """Indicates the kind of an `Image` stored in a file's tag."""

    other = 0
    icon = 1
    other_icon = 2
    front = 3
    back = 4
    leaflet = 5
    media = 6
    lead_artist = 7
    artist = 8
    conductor = 9
    group = 10
    composer = 11
    lyricist = 12
    recording_location = 13
    recording_session = 14
    performance = 15
    screen_capture = 16
    fish = 17
    illustration = 18
    artist_logo = 19
    publisher_logo = 20



# iTunes Sound Check encoding.


import binascii
import codecs
import math
import struct


def sc_decode(soundcheck):
    """Convert a Sound Check bytestring value to a (gain, peak) tuple as
    used by ReplayGain.
    """
    # We decode binary data. If one of the formats gives us a text
    # string, interpret it as UTF-8.
    if isinstance(soundcheck, str):
        soundcheck = soundcheck.encode("utf-8")

    # SoundCheck tags consist of 10 numbers, each represented by 8
    # characters of ASCII hex preceded by a space.
    try:
        soundcheck = codecs.decode(soundcheck.replace(b" ", b""), "hex")
        soundcheck = struct.unpack("!iiiiiiiiii", soundcheck)
    except (struct.error, TypeError, binascii.Error):
        # SoundCheck isn't in the format we expect, so return default
        # values.
        return 0.0, 0.0

    # SoundCheck stores absolute calculated/measured RMS value in an
    # unknown unit. We need to find the ratio of this measurement
    # compared to a reference value of 1000 to get our gain in dB. We
    # play it safe by using the larger of the two values (i.e., the most
    # attenuation).
    maxgain = max(soundcheck[:2])
    if maxgain > 0:
        gain = math.log10(maxgain / 1000.0) * -10
    else:
        # Invalid gain value found.
        gain = 0.0

    # SoundCheck stores peak values as the actual value of the sample,
    # and again separately for the left and right channels. We need to
    # convert this to a percentage of full scale, which is 32768 for a
    # 16 bit sample. Once again, we play it safe by using the larger of
    # the two values.
    peak = max(soundcheck[6:8]) / 32768.0

    return round(gain, 2), round(peak, 6)


def sc_encode(gain, peak):
    """Encode ReplayGain gain/peak values as a Sound Check string."""
    # SoundCheck stores the peak value as the actual value of the
    # sample, rather than the percentage of full scale that RG uses, so
    # we do a simple conversion assuming 16 bit samples.
    peak *= 32768.0

    # SoundCheck stores absolute RMS values in some unknown units rather
    # than the dB values RG uses. We can calculate these absolute values
    # from the gain ratio using a reference value of 1000 units. We also
    # enforce the maximum and minimum value here, which is equivalent to
    # about -18.2dB and 30.0dB.
    g1 = int(min(round((10 ** (gain / -10)) * 1000), 65534)) or 1
    # Same as above, except our reference level is 2500 units.
    g2 = int(min(round((10 ** (gain / -10)) * 2500), 65534)) or 1

    # The purpose of these values are unknown, but they also seem to be
    # unused so we just use zero.
    uk = 0
    values = (g1, g1, g2, g2, uk, uk, int(peak), int(peak), uk, uk)
    return (" %08X" * 10) % values



from .afs import ASFImageStorageStyle, ASFStorageStyle
from .ape import APEv2ImageStorageStyle
from .base import ListStorageStyle, SoundCheckStorageStyleMixin, StorageStyle
from .flac import FlacImageStorageStyle
from .mp3 import (
    MP3DescStorageStyle,
    MP3ImageStorageStyle,
    MP3ListDescStorageStyle,
    MP3ListPeopleStorageStyle,
    MP3ListStorageStyle,
    MP3PeopleStorageStyle,
    MP3SlashPackStorageStyle,
    MP3SoundCheckStorageStyle,
    MP3StorageStyle,
    MP3UFIDStorageStyle,
)
from .mp4 import (
    MP4BoolStorageStyle,
    MP4ImageStorageStyle,
    MP4ListStorageStyle,
    MP4SoundCheckStorageStyle,
    MP4StorageStyle,
    MP4TupleStorageStyle,
)
from .vorbis import VorbisImageStorageStyle

__all__ = [
    "StorageStyle",
    "ListStorageStyle",
    "SoundCheckStorageStyleMixin",
    "ASFStorageStyle",
    "ASFImageStorageStyle",
    "APEv2ImageStorageStyle",
    "FlacImageStorageStyle",
    "MP3StorageStyle",
    "MP3SoundCheckStorageStyle",
    "MP3DescStorageStyle",
    "MP3ListPeopleStorageStyle",
    "MP3PeopleStorageStyle",
    "MP3SlashPackStorageStyle",
    "MP3ImageStorageStyle",
    "MP4TupleStorageStyle",
    "MP3ListStorageStyle",
    "MP3UFIDStorageStyle",
    "MP3ListDescStorageStyle",
    "MP4StorageStyle",
    "MP4BoolStorageStyle",
    "MP4SoundCheckStorageStyle",
    "MP4ImageStorageStyle",
    "MP4ListStorageStyle",
    "VorbisImageStorageStyle",
]



# This file is part of MediaFile.
# Copyright 2016, Adrian Sampson.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

"""Specific, edge-case tests for the MediaFile metadata layer."""

import os
import shutil
import unittest

import mutagen.id3

import mediafile
from test import _common

_sc = mediafile.utils.safe_cast


class EdgeTest(unittest.TestCase):
    def test_emptylist(self):
        # Some files have an ID3 frame that has a list with no elements.
        # This is very hard to produce, so this is just the first 8192
        # bytes of a file found "in the wild".
        emptylist = mediafile.MediaFile(os.path.join(_common.RSRC, b"emptylist.mp3"))
        genre = emptylist.genre
        self.assertEqual(genre, None)

    def test_release_time_with_space(self):
        # Ensures that release times delimited by spaces are ignored.
        # Amie Street produces such files.
        space_time = mediafile.MediaFile(os.path.join(_common.RSRC, b"space_time.mp3"))
        self.assertEqual(space_time.year, 2009)
        self.assertEqual(space_time.month, 9)
        self.assertEqual(space_time.day, 4)

    def test_release_time_with_t(self):
        # Ensures that release times delimited by Ts are ignored.
        # The iTunes Store produces such files.
        t_time = mediafile.MediaFile(os.path.join(_common.RSRC, b"t_time.m4a"))
        self.assertEqual(t_time.year, 1987)
        self.assertEqual(t_time.month, 3)
        self.assertEqual(t_time.day, 31)

    def test_tempo_with_bpm(self):
        # Some files have a string like "128 BPM" in the tempo field
        # rather than just a number.
        f = mediafile.MediaFile(os.path.join(_common.RSRC, b"bpm.mp3"))
        self.assertEqual(f.bpm, 128)

    def test_discc_alternate_field(self):
        # Different taggers use different vorbis comments to reflect
        # the disc and disc count fields: ensure that the alternative
        # style works.
        f = mediafile.MediaFile(os.path.join(_common.RSRC, b"discc.ogg"))
        self.assertEqual(f.disc, 4)
        self.assertEqual(f.disctotal, 5)

    def test_old_ape_version_bitrate(self):
        media_file = os.path.join(_common.RSRC, b"oldape.ape")
        f = mediafile.MediaFile(media_file)
        self.assertEqual(f.bitrate, 0)

    def test_soundcheck_non_ascii(self):
        # Make sure we don't crash when the iTunes SoundCheck field contains
        # non-ASCII binary data.
        f = mediafile.MediaFile(os.path.join(_common.RSRC, b"soundcheck-nonascii.m4a"))
        self.assertEqual(f.rg_track_gain, 0.0)


class InvalidValueToleranceTest(unittest.TestCase):
    def test_safe_cast_string_to_int(self):
        self.assertEqual(_sc(int, "something"), 0)

    def test_safe_cast_string_to_int_with_no_numbers(self):
        self.assertEqual(_sc(int, "-"), 0)

    def test_safe_cast_int_string_to_int(self):
        self.assertEqual(_sc(int, "20"), 20)

    def test_safe_cast_string_to_bool(self):
        self.assertEqual(_sc(bool, "whatever"), False)

    def test_safe_cast_intstring_to_bool(self):
        self.assertEqual(_sc(bool, "5"), True)

    def test_safe_cast_string_to_float(self):
        self.assertAlmostEqual(_sc(float, "1.234"), 1.234)

    def test_safe_cast_int_to_float(self):
        self.assertAlmostEqual(_sc(float, 2), 2.0)

    def test_safe_cast_string_with_cruft_to_float(self):
        self.assertAlmostEqual(_sc(float, "1.234stuff"), 1.234)

    def test_safe_cast_negative_string_to_float(self):
        self.assertAlmostEqual(_sc(float, "-1.234"), -1.234)

    def test_safe_cast_special_chars_to_unicode(self):
        us = _sc(str, "caf\xc3\xa9")
        self.assertTrue(isinstance(us, str))
        self.assertTrue(us.startswith("caf"))

    def test_safe_cast_float_with_no_numbers(self):
        v = _sc(float, "+")
        self.assertEqual(v, 0.0)

    def test_safe_cast_float_with_dot_only(self):
        v = _sc(float, ".")
        self.assertEqual(v, 0.0)

    def test_safe_cast_float_with_multiple_dots(self):
        v = _sc(float, "1.0.0")
        self.assertEqual(v, 1.0)


class SafetyTest(unittest.TestCase, _common.TempDirMixin):
    def setUp(self):
        self.create_temp_dir()

    def tearDown(self):
        self.remove_temp_dir()

    def _exccheck(self, fn, exc, data=""):
        fn = os.path.join(self.temp_dir, fn)
        with open(fn, "w") as f:
            f.write(data)
        try:
            self.assertRaises(exc, mediafile.MediaFile, fn)
        finally:
            os.unlink(fn)  # delete the temporary file

    def test_corrupt_mp3_raises_unreadablefileerror(self):
        # Make sure we catch Mutagen reading errors appropriately.
        self._exccheck(b"corrupt.mp3", mediafile.UnreadableFileError)

    def test_corrupt_mp4_raises_unreadablefileerror(self):
        self._exccheck(b"corrupt.m4a", mediafile.UnreadableFileError)

    def test_corrupt_flac_raises_unreadablefileerror(self):
        self._exccheck(b"corrupt.flac", mediafile.UnreadableFileError)

    def test_corrupt_ogg_raises_unreadablefileerror(self):
        self._exccheck(b"corrupt.ogg", mediafile.UnreadableFileError)

    def test_invalid_ogg_header_raises_unreadablefileerror(self):
        self._exccheck(b"corrupt.ogg", mediafile.UnreadableFileError, "OggS\x01vorbis")

    def test_corrupt_monkeys_raises_unreadablefileerror(self):
        self._exccheck(b"corrupt.ape", mediafile.UnreadableFileError)

    def test_invalid_extension_raises_filetypeerror(self):
        self._exccheck(b"something.unknown", mediafile.FileTypeError)

    def test_magic_xml_raises_unreadablefileerror(self):
        self._exccheck(b"nothing.xml", mediafile.UnreadableFileError, "ftyp")

    @unittest.skipUnless(_common.HAVE_SYMLINK, "platform lacks symlink")
    def test_broken_symlink(self):
        fn = os.path.join(_common.RSRC, b"brokenlink")
        os.symlink("does_not_exist", fn)
        try:
            self.assertRaises(mediafile.UnreadableFileError, mediafile.MediaFile, fn)
        finally:
            os.unlink(fn)


class SideEffectsTest(unittest.TestCase):
    def setUp(self):
        self.empty = os.path.join(_common.RSRC, b"empty.mp3")

    def test_opening_tagless_file_leaves_untouched(self):
        old_mtime = os.stat(self.empty).st_mtime
        mediafile.MediaFile(self.empty)
        new_mtime = os.stat(self.empty).st_mtime
        self.assertEqual(old_mtime, new_mtime)


class MP4EncodingTest(unittest.TestCase, _common.TempDirMixin):
    def setUp(self):
        self.create_temp_dir()
        src = os.path.join(_common.RSRC, b"full.m4a")
        self.path = os.path.join(self.temp_dir, b"test.m4a")
        shutil.copy(src, self.path)

        self.mf = mediafile.MediaFile(self.path)

    def tearDown(self):
        self.remove_temp_dir()

    def test_unicode_label_in_m4a(self):
        self.mf.label = "foo\xe8bar"
        self.mf.save()
        new_mf = mediafile.MediaFile(self.path)
        self.assertEqual(new_mf.label, "foo\xe8bar")


class MP3EncodingTest(unittest.TestCase, _common.TempDirMixin):
    def setUp(self):
        self.create_temp_dir()
        src = os.path.join(_common.RSRC, b"full.mp3")
        self.path = os.path.join(self.temp_dir, b"test.mp3")
        shutil.copy(src, self.path)

        self.mf = mediafile.MediaFile(self.path)

    def test_comment_with_latin1_encoding(self):
        # Set up the test file with a Latin1-encoded COMM frame. The encoding
        # indices defined by MP3 are listed here:
        # https://id3.org/id3v2.4.0-structure
        self.mf.mgfile["COMM::eng"].encoding = 0

        # Try to store non-Latin1 text.
        self.mf.comments = "\u2028"
        self.mf.save()


class ZeroLengthMediaFile(mediafile.MediaFile):
    @property
    def length(self):
        return 0.0


class MissingAudioDataTest(unittest.TestCase):
    def setUp(self):
        super().setUp()
        path = os.path.join(_common.RSRC, b"full.mp3")
        self.mf = ZeroLengthMediaFile(path)

    def test_bitrate_with_zero_length(self):
        del self.mf.mgfile.info.bitrate  # Not available directly.
        self.assertEqual(self.mf.bitrate, 0)


class TypeTest(unittest.TestCase):
    def setUp(self):
        super().setUp()
        path = os.path.join(_common.RSRC, b"full.mp3")
        self.mf = mediafile.MediaFile(path)

    def test_year_integer_in_string(self):
        self.mf.year = "2009"
        self.assertEqual(self.mf.year, 2009)

    def test_set_replaygain_gain_to_none(self):
        self.mf.rg_track_gain = None
        self.assertEqual(self.mf.rg_track_gain, 0.0)

    def test_set_replaygain_peak_to_none(self):
        self.mf.rg_track_peak = None
        self.assertEqual(self.mf.rg_track_peak, 0.0)

    def test_set_year_to_none(self):
        self.mf.year = None
        self.assertIsNone(self.mf.year)

    def test_set_track_to_none(self):
        self.mf.track = None
        self.assertEqual(self.mf.track, 0)

    def test_set_date_to_none(self):
        self.mf.date = None
        self.assertIsNone(self.mf.date)
        self.assertIsNone(self.mf.year)
        self.assertIsNone(self.mf.month)
        self.assertIsNone(self.mf.day)


class SoundCheckTest(unittest.TestCase):
    def test_round_trip(self):
        data = mediafile.utils.sc_encode(1.0, 1.0)
        gain, peak = mediafile.utils.sc_decode(data)
        self.assertEqual(gain, 1.0)
        self.assertEqual(peak, 1.0)

    def test_decode_zero(self):
        data = (
            b" 80000000 80000000 00000000 00000000 00000000 00000000 "
            b"00000000 00000000 00000000 00000000"
        )
        gain, peak = mediafile.utils.sc_decode(data)
        self.assertEqual(gain, 0.0)
        self.assertEqual(peak, 0.0)

    def test_malformatted(self):
        gain, peak = mediafile.utils.sc_decode(b"foo")
        self.assertEqual(gain, 0.0)
        self.assertEqual(peak, 0.0)

    def test_special_characters(self):
        gain, peak = mediafile.utils.sc_decode("caf\xe9".encode())
        self.assertEqual(gain, 0.0)
        self.assertEqual(peak, 0.0)

    def test_decode_handles_unicode(self):
        # Most of the time, we expect to decode the raw bytes. But some formats
        # might give us text strings, which we need to handle.
        gain, peak = mediafile.utils.sc_decode("caf\xe9")
        self.assertEqual(gain, 0.0)
        self.assertEqual(peak, 0.0)

    def test_encode_excessive_gain(self):
        # The mimimum value of SoundCheck gain is 30.0dB.
        data = mediafile.utils.sc_encode(60.0, 1.0)
        gain, _ = mediafile.utils.sc_decode(data)
        self.assertEqual(gain, 30.0)


class ID3v23Test(unittest.TestCase, _common.TempDirMixin):
    def _make_test(self, ext=b"mp3", id3v23=False):
        self.create_temp_dir()
        src = os.path.join(_common.RSRC, b"full." + ext)
        self.path = os.path.join(self.temp_dir, b"test." + ext)
        shutil.copy(src, self.path)
        return mediafile.MediaFile(self.path, id3v23=id3v23)

    def _delete_test(self):
        self.remove_temp_dir()

    def test_v24_year_tag(self):
        mf = self._make_test(id3v23=False)
        try:
            mf.year = 2013
            mf.save()
            frame = mf.mgfile["TDRC"]
            self.assertTrue("2013" in str(frame))
            self.assertTrue("TYER" not in mf.mgfile)
        finally:
            self._delete_test()

    def test_v23_year_tag(self):
        mf = self._make_test(id3v23=True)
        try:
            mf.year = 2013
            mf.save()
            frame = mf.mgfile["TYER"]
            self.assertTrue("2013" in str(frame))
            self.assertTrue("TDRC" not in mf.mgfile)
        finally:
            self._delete_test()

    def test_v23_on_non_mp3_is_noop(self):
        mf = self._make_test(b"m4a", id3v23=True)
        try:
            mf.year = 2013
            mf.save()
        finally:
            self._delete_test()

    def test_image_encoding(self):
        """For compatibility with OS X/iTunes.

        See https://github.com/beetbox/beets/issues/899#issuecomment-62437773
        """

        for v23 in [True, False]:
            mf = self._make_test(id3v23=v23)
            try:
                mf.images = [
                    mediafile.Image(b"data", desc=""),
                    mediafile.Image(b"data", desc="foo"),
                    mediafile.Image(b"data", desc="\u0185"),
                ]
                mf.save()
                apic_frames = mf.mgfile.tags.getall("APIC")
                encodings = {f.desc: f.encoding for f in apic_frames}
                self.assertEqual(
                    encodings,
                    {
                        "": mutagen.id3.Encoding.LATIN1,
                        "foo": mutagen.id3.Encoding.LATIN1,
                        "\u0185": mutagen.id3.Encoding.UTF16,
                    },
                )
            finally:
                self._delete_test()


class ReadOnlyTagTest(unittest.TestCase, _common.TempDirMixin):
    def setUp(self):
        self.create_temp_dir()
        self.key = "READ_ONLY_TEST"
        self.field = mediafile.MediaField(
            mediafile.MP3StorageStyle(self.key, read_only=True),
            mediafile.MP4StorageStyle(
                "----:com.apple.iTunes:" + self.key, read_only=True
            ),
            mediafile.StorageStyle(self.key, read_only=True),
            mediafile.ASFStorageStyle(self.key, read_only=True),
        )

        if "read_only_test" not in mediafile.MediaFile.fields():
            mediafile.MediaFile.add_field("read_only_test", self.field)

    def test_read(self):
        path = os.path.join(_common.RSRC, b"empty.flac")
        mf = mediafile.MediaFile(path)
        mf.mgfile.tags[self.key] = "don't"
        self.assertEqual("don't", mf.read_only_test)

    def test_write(self):
        src = os.path.join(_common.RSRC, b"empty.flac")
        path = os.path.join(self.temp_dir, b"test.flac")
        shutil.copy(src, path)
        mf = mediafile.MediaFile(path)
        mf.read_only_field = "something terrible"
        mf.save()
        self.assertNotIn(self.key, mf.mgfile.tags)

    def tearDown(self):
        self.remove_temp_dir()
        try:
            del mediafile.MediaFile.read_only_test
        except AttributeError:
            pass


def suite():
    return unittest.TestLoader().loadTestsFromName(__name__)


if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# This file is part of mediafile.
# Copyright 2016, Adrian Sampson.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

"""Some common functionality for mediafile tests."""

import os
import shutil
import sys
import tempfile

# Test resources path.
RSRC = os.path.join(os.path.dirname(__file__), "rsrc").encode("utf8")

# OS feature test.
HAVE_SYMLINK = sys.platform != "win32"


# Convenience methods for setting up a temporary sandbox directory for tests
# that need to interact with the filesystem.
class TempDirMixin:
    """Text mixin for creating and deleting a temporary directory."""

    def create_temp_dir(self):
        """Create a temporary directory and assign it into `self.temp_dir`.
        Call `remove_temp_dir` later to delete it.
        """
        path = tempfile.mkdtemp()
        if not isinstance(path, bytes):
            path = path.encode("utf8")
        self.temp_dir = path

    def remove_temp_dir(self):
        """Delete the temporary directory created by `create_temp_dir`."""
        if os.path.isdir(self.temp_dir):
            shutil.rmtree(self.temp_dir)
